@@ -11,11 +11,17 @@ from sqlparse import tokens
 
 
 def is_keyword(value):
+    """Checks for a keyword.
+
+    If the given value is in one of the KEYWORDS_* dictionary
+    it's considered a keyword. Otherwise tokens.Name is returned.
+    """
     val = value.upper()
     return (KEYWORDS_COMMON.get(val)
             or KEYWORDS_ORACLE.get(val)
             or KEYWORDS_PLPGSQL.get(val)
             or KEYWORDS_HQL.get(val)
+            or KEYWORDS_MSACCESS.get(val)
             or KEYWORDS.get(val, tokens.Name)), value
 
 
@@ -56,13 +62,13 @@ SQL_REGEX = {
         # see issue #39
         # Spaces around period `schema . name` are valid identifier
         # TODO: Spaces before period not implemented
-        (r'[A-ZÀ-Ü]\w*(?=\s*\.)', tokens.Name),  # 'Name'   .
+        (r'[A-ZÀ-Ü]\w*(?=\s*\.)', tokens.Name),  # 'Name'.
         # FIXME(atronah): never match,
         # because `re.match` doesn't work with look-behind regexp feature
         (r'(?<=\.)[A-ZÀ-Ü]\w*', tokens.Name),  # .'Name'
         (r'[A-ZÀ-Ü]\w*(?=\()', tokens.Name),  # side effect: change kw to func
         (r'-?0x[\dA-F]+', tokens.Number.Hexadecimal),
-        (r'-?\d*(\.\d+)?E-?\d+', tokens.Number.Float),
+        (r'-?\d+(\.\d+)?E-?\d+', tokens.Number.Float),
         (r'(?![_A-ZÀ-Ü])-?(\d+(\.\d*)|\.\d+)(?![_A-ZÀ-Ü])',
          tokens.Number.Float),
         (r'(?![_A-ZÀ-Ü])-?\d+(?![_A-ZÀ-Ü])', tokens.Number.Integer),
@@ -90,10 +96,13 @@ SQL_REGEX = {
          tokens.Keyword),
         (r"(AT|WITH')\s+TIME\s+ZONE\s+'[^']+'", tokens.Keyword.TZCast),
         (r'(NOT\s+)?(LIKE|ILIKE|RLIKE)\b', tokens.Operator.Comparison),
-        (r'[0-9_A-ZÀ-Ü][_$#\w]*', is_keyword),
+        (r'(NOT\s+)?(REGEXP)\b', tokens.Operator.Comparison),
+        # Check for keywords, also returns tokens.Name if regex matches
+        # but the match isn't a keyword.
+        (r'[0-9_\w][_$#\w]*', is_keyword),
         (r'[;:()\[\],\.]', tokens.Punctuation),
         (r'[<>=~!]+', tokens.Operator.Comparison),
-        (r'[+/@#%^&|`?^-]+', tokens.Operator),
+        (r'[+/@#%^&|^-]+', tokens.Operator),
     ]}
 
 FLAGS = re.IGNORECASE | re.UNICODE
@@ -241,6 +250,7 @@ KEYWORDS = {
     'DISABLE': tokens.Keyword,
     'DISCONNECT': tokens.Keyword,
     'DISPATCH': tokens.Keyword,
+    'DIV': tokens.Operator,
     'DO': tokens.Keyword,
     'DOMAIN': tokens.Keyword,
     'DYNAMIC': tokens.Keyword,
@@ -314,7 +324,7 @@ KEYWORDS = {
     'INCREMENT': tokens.Keyword,
     'INDEX': tokens.Keyword,
 
-    'INDITCATOR': tokens.Keyword,
+    'INDICATOR': tokens.Keyword,
     'INFIX': tokens.Keyword,
     'INHERITS': tokens.Keyword,
     'INITIAL': tokens.Keyword,
@@ -745,6 +755,7 @@ KEYWORDS_ORACLE = {
     'DOUBLE': tokens.Keyword,
     'DUMP': tokens.Keyword,
 
+    'ELSIF': tokens.Keyword,
     'EVENTS': tokens.Keyword,
     'EXCEPTIONS': tokens.Keyword,
     'EXPLAIN': tokens.Keyword,
@@ -833,6 +844,7 @@ KEYWORDS_ORACLE = {
 
 # PostgreSQL Syntax
 KEYWORDS_PLPGSQL = {
+    'CONFLICT': tokens.Keyword,
     'WINDOW': tokens.Keyword,
     'PARTITION': tokens.Keyword,
     'OVER': tokens.Keyword,
@@ -841,6 +853,7 @@ KEYWORDS_PLPGSQL = {
     'PLPGSQL': tokens.Keyword,
     'INHERIT': tokens.Keyword,
     'INDEXES': tokens.Keyword,
+    'ON_ERROR_STOP': tokens.Keyword,
 
     'BYTEA': tokens.Keyword,
     'BIGSERIAL': tokens.Keyword,
@@ -904,6 +917,7 @@ KEYWORDS_HQL = {
     'INLINE': tokens.Keyword,
     'INSTR': tokens.Keyword,
     'LEN': tokens.Keyword,
+    'MAP': tokens.Name.Builtin,
     'MAXELEMENT': tokens.Keyword,
     'MAXINDEX': tokens.Keyword,
     'MAX_PART_DATE': tokens.Keyword,
@@ -935,9 +949,12 @@ KEYWORDS_HQL = {
     'SQRT': tokens.Keyword,
     'STACK': tokens.Keyword,
     'STR': tokens.Keyword,
+    'STRING': tokens.Name.Builtin,
+    'STRUCT': tokens.Name.Builtin,
     'SUBSTR': tokens.Keyword,
     'SUMMARY': tokens.Keyword,
     'TBLPROPERTIES': tokens.Keyword,
+    'TIMESTAMP': tokens.Name.Builtin,
     'TIMESTAMP_ISO': tokens.Keyword,
     'TO_CHAR': tokens.Keyword,
     'TO_DATE': tokens.Keyword,
@@ -953,3 +970,8 @@ KEYWORDS_HQL = {
     'BREAK': tokens.Keyword,
     'LEAVE': tokens.Keyword,
 }
+
+
+KEYWORDS_MSACCESS = {
+    'DISTINCTROW': tokens.Keyword,
+}
